@@ -21,7 +21,6 @@ private Q_SLOTS:
     void testName();
     void testPointerButton();
     void testPointerPos();
-    void testDestroyThroughTerminate();
     void testRepeatInfo();
     void testMultiple();
 };
@@ -31,9 +30,9 @@ static const QString s_socketName = QStringLiteral("kwin-wayland-server-seat-tes
 void TestWaylandServerSeat::testCapabilities()
 {
     Display display;
-    display.setSocketName(s_socketName);
+    display.addSocketName(s_socketName);
     display.start();
-    SeatInterface *seat = display.createSeat();
+    SeatInterface *seat = new SeatInterface(&display);
     QVERIFY(!seat->hasKeyboard());
     QVERIFY(!seat->hasPointer());
     QVERIFY(!seat->hasTouch());
@@ -81,9 +80,9 @@ void TestWaylandServerSeat::testCapabilities()
 void TestWaylandServerSeat::testName()
 {
     Display display;
-    display.setSocketName(s_socketName);
+    display.addSocketName(s_socketName);
     display.start();
-    SeatInterface *seat = display.createSeat();
+    SeatInterface *seat = new SeatInterface(&display);
     QCOMPARE(seat->name(), QString());
 
     QSignalSpy nameSpy(seat, SIGNAL(nameChanged(QString)));
@@ -100,9 +99,9 @@ void TestWaylandServerSeat::testName()
 void TestWaylandServerSeat::testPointerButton()
 {
     Display display;
-    display.setSocketName(s_socketName);
+    display.addSocketName(s_socketName);
     display.start();
-    SeatInterface *seat = display.createSeat();
+    SeatInterface *seat = new SeatInterface(&display);
     PointerInterface *pointer = seat->focusedPointer();
     QVERIFY(!pointer);
 
@@ -130,9 +129,9 @@ void TestWaylandServerSeat::testPointerButton()
 void TestWaylandServerSeat::testPointerPos()
 {
     Display display;
-    display.setSocketName(s_socketName);
+    display.addSocketName(s_socketName);
     display.start();
-    SeatInterface *seat = display.createSeat();
+    SeatInterface *seat = new SeatInterface(&display);
     QSignalSpy seatPosSpy(seat, SIGNAL(pointerPosChanged(QPointF)));
     QVERIFY(seatPosSpy.isValid());
     PointerInterface *pointer = seat->focusedPointer();
@@ -155,49 +154,38 @@ void TestWaylandServerSeat::testPointerPos()
     QCOMPARE(seatPosSpy.last().first().toPointF(), QPointF(5, 7));
 }
 
-void TestWaylandServerSeat::testDestroyThroughTerminate()
-{
-    QScopedPointer<Display> display(new Display());
-    display->setSocketName(s_socketName);
-    display->start();
-    SeatInterface *seat = display->createSeat();
-    QSignalSpy destroyedSpy(seat, SIGNAL(destroyed(QObject*)));
-    QVERIFY(destroyedSpy.isValid());
-    display.reset();
-    QVERIFY(!destroyedSpy.isEmpty());
-}
-
 void TestWaylandServerSeat::testRepeatInfo()
 {
     Display display;
-    display.setSocketName(s_socketName);
+    display.addSocketName(s_socketName);
     display.start();
-    SeatInterface *seat = display.createSeat();
-    QCOMPARE(seat->keyRepeatRate(), 0);
-    QCOMPARE(seat->keyRepeatDelay(), 0);
-    seat->setKeyRepeatInfo(25, 660);
-    QCOMPARE(seat->keyRepeatRate(), 25);
-    QCOMPARE(seat->keyRepeatDelay(), 660);
+    SeatInterface *seat = new SeatInterface(&display);
+    seat->setHasKeyboard(true);
+    QCOMPARE(seat->keyboard()->keyRepeatRate(), 0);
+    QCOMPARE(seat->keyboard()->keyRepeatDelay(), 0);
+    seat->keyboard()->setRepeatInfo(25, 660);
+    QCOMPARE(seat->keyboard()->keyRepeatRate(), 25);
+    QCOMPARE(seat->keyboard()->keyRepeatDelay(), 660);
     // setting negative values should result in 0
-    seat->setKeyRepeatInfo(-25, -660);
-    QCOMPARE(seat->keyRepeatRate(), 0);
-    QCOMPARE(seat->keyRepeatDelay(), 0);
+    seat->keyboard()->setRepeatInfo(-25, -660);
+    QCOMPARE(seat->keyboard()->keyRepeatRate(), 0);
+    QCOMPARE(seat->keyboard()->keyRepeatDelay(), 0);
 }
 
 void TestWaylandServerSeat::testMultiple()
 {
     Display display;
-    display.setSocketName(s_socketName);
+    display.addSocketName(s_socketName);
     display.start();
     QVERIFY(display.seats().isEmpty());
-    SeatInterface *seat1 = display.createSeat();
+    SeatInterface *seat1 = new SeatInterface(&display);
     QCOMPARE(display.seats().count(), 1);
     QCOMPARE(display.seats().at(0), seat1);
-    SeatInterface *seat2 = display.createSeat();
+    SeatInterface *seat2 = new SeatInterface(&display);
     QCOMPARE(display.seats().count(), 2);
     QCOMPARE(display.seats().at(0), seat1);
     QCOMPARE(display.seats().at(1), seat2);
-    SeatInterface *seat3 = display.createSeat();
+    SeatInterface *seat3 = new SeatInterface(&display);
     QCOMPARE(display.seats().count(), 3);
     QCOMPARE(display.seats().at(0), seat1);
     QCOMPARE(display.seats().at(1), seat2);
