@@ -8,6 +8,7 @@
 #define WAYLAND_SERVER_SURFACE_INTERFACE_P_H
 
 #include "surface_interface.h"
+#include "utils.h"
 // Qt
 #include <QHash>
 #include <QVector>
@@ -41,7 +42,7 @@ public:
         QRegion damage = QRegion();
         QRegion bufferDamage = QRegion();
         QRegion opaque = QRegion();
-        QRegion input = QRegion();
+        QRegion input = infiniteRegion();
         QRectF sourceGeometry = QRectF();
         QSize destinationSize = QSize();
         QSize size = QSize();
@@ -54,7 +55,6 @@ public:
         bool blurIsSet = false;
         bool contrastIsSet = false;
         bool slideIsSet = false;
-        bool inputIsInfinite = true;
         bool childrenChanged = false;
         bool bufferScaleIsSet = false;
         bool bufferTransformIsSet = false;
@@ -64,7 +64,7 @@ public:
         QPoint offset = QPoint();
         BufferInterface *buffer = nullptr;
         // stacking order: bottom (first) -> top (last)
-        QList<QPointer<SubSurfaceInterface>> children;
+        QList<SubSurfaceInterface *> children;
         QPointer<ShadowInterface> shadow;
         QPointer<BlurInterface> blur;
         QPointer<ContrastInterface> contrast;
@@ -76,21 +76,21 @@ public:
     explicit SurfaceInterfacePrivate(SurfaceInterface *q);
     ~SurfaceInterfacePrivate() override;
 
-    void addChild(QPointer<SubSurfaceInterface> subsurface);
-    void removeChild(QPointer<SubSurfaceInterface> subsurface);
-    bool raiseChild(QPointer<SubSurfaceInterface> subsurface, SurfaceInterface *sibling);
-    bool lowerChild(QPointer<SubSurfaceInterface> subsurface, SurfaceInterface *sibling);
+    void addChild(SubSurfaceInterface *subsurface);
+    void removeChild(SubSurfaceInterface *subsurface);
+    bool raiseChild(SubSurfaceInterface *subsurface, SurfaceInterface *sibling);
+    bool lowerChild(SubSurfaceInterface *subsurface, SurfaceInterface *sibling);
     void setShadow(const QPointer<ShadowInterface> &shadow);
     void setBlur(const QPointer<BlurInterface> &blur);
     void setContrast(const QPointer<ContrastInterface> &contrast);
     void setSlide(const QPointer<SlideInterface> &slide);
-    void installPointerConstraint(LockedPointerInterface *lock);
-    void installPointerConstraint(ConfinedPointerInterface *confinement);
+    void installPointerConstraint(LockedPointerV1Interface *lock);
+    void installPointerConstraint(ConfinedPointerV1Interface *confinement);
     void installIdleInhibitor(IdleInhibitorV1Interface *inhibitor);
 
-    void commitSubSurface();
     void commit();
     QMatrix4x4 buildSurfaceToBufferMatrix(const State *state);
+    void swapStates(State *source, State *target, bool emitChanged);
 
     CompositorInterface *compositor;
     SurfaceInterface *q;
@@ -99,11 +99,12 @@ public:
     State current;
     State pending;
     State cached;
-    QPointer<SubSurfaceInterface> subSurface;
+    SubSurfaceInterface *subSurface = nullptr;
     QRegion trackedDamage;
     QMatrix4x4 surfaceToBufferMatrix;
     QMatrix4x4 bufferToSurfaceMatrix;
     QSize bufferSize;
+    QRegion inputRegion;
 
     // workaround for https://bugreports.qt.io/browse/QTBUG-52192
     // A subsurface needs to be considered mapped even if it doesn't have a buffer attached
@@ -113,9 +114,11 @@ public:
 
     QVector<OutputInterface *> outputs;
 
-    QPointer<LockedPointerInterface> lockedPointer;
-    QPointer<ConfinedPointerInterface> confinedPointer;
+    LockedPointerV1Interface *lockedPointer = nullptr;
+    ConfinedPointerV1Interface *confinedPointer = nullptr;
     QHash<OutputInterface*, QMetaObject::Connection> outputDestroyedConnections;
+    QHash<OutputInterface*, QMetaObject::Connection> outputBoundConnections;
+
     QVector<IdleInhibitorV1Interface*> idleInhibitors;
     ViewportInterface *viewportExtension = nullptr;
     SurfaceInterface *dataProxy = nullptr;
@@ -138,8 +141,6 @@ protected:
     void surface_damage_buffer(Resource *resource, int32_t x, int32_t y, int32_t width, int32_t height) override;
 
 private:
-    void swapStates(State *source, State *target, bool emitChanged);
-
     QMetaObject::Connection constrainsOneShotConnection;
     QMetaObject::Connection constrainsUnboundConnection;
 };
