@@ -10,13 +10,14 @@
 #include "display.h"
 #include "output_interface.h"
 #include "seat_interface.h"
+#include "utils.h"
 
 #include <QTimer>
 
 namespace KWaylandServer
 {
 
-static const int s_version = 2;
+static const int s_version = 3;
 
 XdgShellInterfacePrivate::XdgShellInterfacePrivate(XdgShellInterface *shell)
     : q(shell)
@@ -186,9 +187,11 @@ void XdgSurfaceInterfacePrivate::xdg_surface_destroy(Resource *resource)
 
 void XdgSurfaceInterfacePrivate::xdg_surface_get_toplevel(Resource *resource, uint32_t id)
 {
-    if (SurfaceRole::get(surface)) {
+    const SurfaceRole *surfaceRole = SurfaceRole::get(surface);
+    if (surfaceRole) {
         wl_resource_post_error(resource->handle, error_already_constructed,
-                               "xdg_surface has already been constructured");
+                               "the surface already has a role assigned %s",
+                               surfaceRole->name().constData());
         return;
     }
 
@@ -203,15 +206,19 @@ void XdgSurfaceInterfacePrivate::xdg_surface_get_popup(Resource *resource, uint3
                                                        ::wl_resource *parentResource,
                                                        ::wl_resource *positionerResource)
 {
-    if (SurfaceRole::get(surface)) {
+    const SurfaceRole *surfaceRole = SurfaceRole::get(surface);
+    if (surfaceRole) {
         wl_resource_post_error(resource->handle, error_already_constructed,
-                               "xdg_surface has already been constructured");
+                               "the surface already has a role assigned %s",
+                               surfaceRole->name().constData());
         return;
     }
 
     XdgPositioner positioner = XdgPositioner::get(positionerResource);
     if (!positioner.isComplete()) {
-        wl_resource_post_error(resource->handle, QtWaylandServer::xdg_wm_base::error_invalid_positioner,
+        auto shellPrivate = XdgShellInterfacePrivate::get(shell);
+        wl_resource_post_error(shellPrivate->resourceForXdgSurface(q)->handle,
+                               QtWaylandServer::xdg_wm_base::error_invalid_positioner,
                                "xdg_positioner is incomplete");
         return;
     }
@@ -298,14 +305,15 @@ QRect XdgSurfaceInterface::windowGeometry() const
 
 XdgSurfaceInterface *XdgSurfaceInterface::get(::wl_resource *resource)
 {
-    if (auto surface = QtWaylandServer::xdg_surface::Resource::fromResource(resource))
-        return static_cast<XdgSurfaceInterfacePrivate *>(surface->object())->q;
+    if (auto surfacePrivate = resource_cast<XdgSurfaceInterfacePrivate *>(resource)) {
+        return surfacePrivate->q;
+    }
     return nullptr;
 }
 
 XdgToplevelInterfacePrivate::XdgToplevelInterfacePrivate(XdgToplevelInterface *toplevel,
                                                          XdgSurfaceInterface *surface)
-    : SurfaceRole(surface->surface())
+    : SurfaceRole(surface->surface(), QByteArrayLiteral("xdg_toplevel"))
     , q(toplevel)
     , xdgSurface(surface)
 {
@@ -509,9 +517,7 @@ XdgToplevelInterfacePrivate *XdgToplevelInterfacePrivate::get(XdgToplevelInterfa
 
 XdgToplevelInterfacePrivate *XdgToplevelInterfacePrivate::get(wl_resource *resource)
 {
-    if (auto toplevel = QtWaylandServer::xdg_toplevel::Resource::fromResource(resource))
-        return static_cast<XdgToplevelInterfacePrivate *>(toplevel->object());
-    return nullptr;
+    return resource_cast<XdgToplevelInterfacePrivate *>(resource);
 }
 
 XdgToplevelInterface::XdgToplevelInterface(XdgSurfaceInterface *surface, ::wl_resource *resource)
@@ -624,8 +630,8 @@ void XdgToplevelInterface::sendClose()
 
 XdgToplevelInterface *XdgToplevelInterface::get(::wl_resource *resource)
 {
-    if (auto toplevel = QtWaylandServer::xdg_toplevel::Resource::fromResource(resource)) {
-        return static_cast<XdgToplevelInterfacePrivate *>(toplevel->object())->q;
+    if (auto toplevelPrivate = resource_cast<XdgToplevelInterfacePrivate *>(resource)) {
+        return toplevelPrivate->q;
     }
     return nullptr;
 }
@@ -637,7 +643,7 @@ XdgPopupInterfacePrivate *XdgPopupInterfacePrivate::get(XdgPopupInterface *popup
 
 XdgPopupInterfacePrivate::XdgPopupInterfacePrivate(XdgPopupInterface *popup,
                                                    XdgSurfaceInterface *surface)
-    : SurfaceRole(surface->surface())
+    : SurfaceRole(surface->surface(), QByteArrayLiteral("xdg_popup"))
     , q(popup)
     , xdgSurface(surface)
 {
@@ -699,6 +705,13 @@ void XdgPopupInterfacePrivate::xdg_popup_grab(Resource *resource, ::wl_resource
     emit q->grabRequested(seat, serial);
 }
 
+void XdgPopupInterfacePrivate::xdg_popup_reposition(Resource *resource, ::wl_resource *positionerResource, uint32_t token)
+{
+    Q_UNUSED(resource)
+    positioner = XdgPositioner::get(positionerResource);
+    emit q->repositionRequested(token);
+}
+
 XdgPopupInterface::XdgPopupInterface(XdgSurfaceInterface *surface,
                                      SurfaceInterface *parentSurface,
                                      const XdgPositioner &positioner,
@@ -757,10 +770,17 @@ void XdgPopupInterface::sendPopupDone()
     d->send_popup_done();
 }
 
+void XdgPopupInterface::sendRepositioned(quint32 token)
+{
+    Q_ASSERT(d->resource()->version() >= XDG_POPUP_REPOSITIONED_SINCE_VERSION);
+    d->send_repositioned(token);
+}
+
 XdgPopupInterface *XdgPopupInterface::get(::wl_resource *resource)
 {
-    if (auto popup = QtWaylandServer::xdg_popup::Resource::fromResource(resource))
-        return static_cast<XdgPopupInterfacePrivate *>(popup->object())->q;
+    if (auto popupPrivate = resource_cast<XdgPopupInterfacePrivate *>(resource)) {
+        return popupPrivate->q;
+    }
     return nullptr;
 }
 
@@ -772,9 +792,7 @@ XdgPositionerPrivate::XdgPositionerPrivate(::wl_resource *resource)
 
 XdgPositionerPrivate *XdgPositionerPrivate::get(wl_resource *resource)
 {
-    if (auto positioner = QtWaylandServer::xdg_positioner::Resource::fromResource(resource))
-        return static_cast<XdgPositionerPrivate *>(positioner->object());
-    return nullptr;
+    return resource_cast<XdgPositionerPrivate *>(resource);
 }
 
 void XdgPositionerPrivate::xdg_positioner_destroy_resource(Resource *resource)
@@ -847,6 +865,24 @@ void XdgPositionerPrivate::xdg_positioner_set_anchor(Resource *resource, uint32_
     }
 }
 
+void XdgPositionerPrivate::xdg_positioner_set_parent_size(Resource *resource, int32_t width, int32_t height)
+{
+    Q_UNUSED(resource)
+    data->parentSize = QSize(width, height);
+}
+
+void XdgPositionerPrivate::xdg_positioner_set_reactive(Resource *resource)
+{
+    Q_UNUSED(resource)
+    data->isReactive = true;
+}
+
+void XdgPositionerPrivate::xdg_positioner_set_parent_configure(Resource *resource, uint32_t serial)
+{
+    Q_UNUSED(resource)
+    data->parentConfigure = serial;
+}
+
 void XdgPositionerPrivate::xdg_positioner_set_gravity(Resource *resource, uint32_t gravity)
 {
     if (gravity > gravity_bottom_right) {
@@ -998,6 +1034,21 @@ QPoint XdgPositioner::offset() const
     return d->offset;
 }
 
+QSize XdgPositioner::parentSize() const
+{
+    return d->parentSize;
+}
+
+bool XdgPositioner::isReactive() const
+{
+    return d->isReactive;
+}
+
+quint32 XdgPositioner::parentConfigure() const
+{
+    return d->parentConfigure;
+}
+
 XdgPositioner XdgPositioner::get(::wl_resource *resource)
 {
     XdgPositionerPrivate *xdgPositionerPrivate = XdgPositionerPrivate::get(resource);
